package cz.muni.fi.pb162.cviceni;
import java.util.Map;
import java.util.Set;

/**
 * Rozhrani Graph reprezentuje obycejny graf 
 * (obycejny znamena, ze mezi dvema uzly muze byt maximalne jedna hrana).
 * @author Zuzana Neverilova
 * @version 2. 12. 2005
 */

public interface Graph
{
    /**
     * Prida uzel daneho jmena
     * @param name jmeno uzlu
     * @return uzel, ktery se pridal
     */
    Node addNode(String name);
    /**
     * Prida uzel
     * @param node uzel
     * @throws IllegalArgumentException, pokud je uzel uz v grafu
     * @return uzel, ktery se pridal
     */    
    Node addNode(Node node);
    /**
     * Ziska uzel podle jmena
     * @param name jmeno uzlu
     * @return uzel nebo null, pokud uzel takoveho jmena neexistuje
     */
    Node getNode(String name);
    /**
     * Prida hranu
     * @param edge hrana, ktera se prida
     * @throws NullPointerException, pokud hrana nema uzly v grafu
     * @return hrana, ktera se pridala
     */
    Edge addEdge(Edge e);
    /**
     * Vytvori a prida hranu mezi 2 uzly o zadane delce. Pokud nektery z uzlu v grafu neni, prida se.
     * @param from jmeno uzlu, ze ktereho hrana vychazi
     * @param to jmeno uzlu, kam hrana smeruje
     * @param length delka hrany
     * @return pridana hrana
     */    
    Edge addEdge(String from, String to, int length);
    /**
     * Vrati mnozinu hran v grafu, ktere vedou ze zadaneho uzlu.
     * @param from uzel, ze ktereho hrany vychazeji
     * @return mnozina hran
     */
    Set<Edge> getEdges(Node from);
    /**
     * Prida vsechny uzly z kontejneru.
     * @param nodes kontejner<String, Node> s uzly
     */
    void addNodes(Map<String, Node> nodes);
    /**
     * Vrati true, pokud existuje cesta z jednoho uzlu do druheho.
     * @param from uzel, ze ktereho vychazime
     * @param to uzel, do ktereho vchazime
     * @return true, pokud cesta existuje, jinak false
     * @throws NullPointerException, pokud je nektery zadany uzel null
     * @throws IllegalArgumentException, pokud nektery uzel neexistuje
     */
    boolean existsPath(String from, String to);
    /**
     * Textova reprezentace grafu. Vypisou se hrany (mohly by se vypsat i uzly).
     * @return textova reprezentace grafu
     */
    String toString();
    /**
     * Vrati vsechny hrany grafu.
     * @return mnozina vsech hran
     */
    Set<Edge> getEdges();    
    /**
     * Vrati graf, ktery je minimalni kostrou grafu. Alogoritmus nalezeni minimalni kostry
     * muze byt Kruskaluv-Boruvkuv nebo Jarnikuv-Primuv.
     * @return minimalni kostra
     */    
    Graph minimumSpanTree();
}
